@@ -1,9 +1,11 @@
-from cStringIO import StringIO
-
 from webtest import TestApp
+from six import b as b_
+from six import u as u_
+from six.moves import cStringIO as StringIO
+
+from webob import Response
 
 from pecan import make_app, expose, redirect, abort
-from pecan.core import state
 from pecan.hooks import (
     PecanHook, TransactionHook, HookController, RequestViewerHook
 )
@@ -39,7 +41,7 @@ class TestHooks(PecanTestCase):
         app = TestApp(make_app(RootController(), hooks=[SimpleHook()]))
         response = app.get('/')
         assert response.status_int == 200
-        assert response.body == 'Hello, World!'
+        assert response.body == b_('Hello, World!')
 
         assert len(run_hook) == 4
         assert run_hook[0] == 'on_route'
@@ -77,7 +79,7 @@ class TestHooks(PecanTestCase):
         ]))
         response = app.get('/')
         assert response.status_int == 200
-        assert response.body == 'Hello, World!'
+        assert response.body == b_('Hello, World!')
 
         assert len(run_hook) == 10
         assert run_hook[0] == 'on_route1'
@@ -118,7 +120,7 @@ class TestHooks(PecanTestCase):
 
         response = app.get('/')
         assert response.status_int == 200
-        assert response.body == 'Hello World!'
+        assert response.body == b_('Hello World!')
 
         assert len(run_hook) == 2
         assert run_hook[0] == 'on_route'
@@ -127,13 +129,40 @@ class TestHooks(PecanTestCase):
         run_hook = []
         try:
             response = app.get('/causeerror')
-        except Exception, e:
+        except Exception as e:
             assert isinstance(e, IndexError)
 
         assert len(run_hook) == 2
         assert run_hook[0] == 'on_route'
         assert run_hook[1] == 'error'
 
+    def test_on_error_response_hook(self):
+        run_hook = []
+
+        class RootController(object):
+            @expose()
+            def causeerror(self):
+                return [][1]
+
+        class ErrorHook(PecanHook):
+            def on_error(self, state, e):
+                run_hook.append('error')
+
+                r = Response()
+                r.text = u_('on_error')
+
+                return r
+
+        app = TestApp(make_app(RootController(), hooks=[
+            ErrorHook()
+        ]))
+
+        response = app.get('/causeerror')
+
+        assert len(run_hook) == 1
+        assert run_hook[0] == 'error'
+        assert response.text == 'on_error'
+
     def test_prioritized_hooks(self):
         run_hook = []
 
@@ -167,7 +196,7 @@ class TestHooks(PecanTestCase):
         app = TestApp(papp)
         response = app.get('/')
         assert response.status_int == 200
-        assert response.body == 'Hello, World!'
+        assert response.body == b_('Hello, World!')
 
         assert len(run_hook) == 10
         assert run_hook[0] == 'on_route3'
@@ -224,7 +253,7 @@ class TestHooks(PecanTestCase):
         app = TestApp(make_app(RootController()))
         response = app.get('/')
         assert response.status_int == 200
-        assert response.body == 'Hello, World!'
+        assert response.body == b_('Hello, World!')
 
         assert len(run_hook) == 1
         assert run_hook[0] == 'inside'
@@ -233,7 +262,7 @@ class TestHooks(PecanTestCase):
 
         response = app.get('/sub/')
         assert response.status_int == 200
-        assert response.body == 'Inside here!'
+        assert response.body == b_('Inside here!')
 
         assert len(run_hook) == 3
         assert run_hook[0] == 'before'
@@ -243,7 +272,7 @@ class TestHooks(PecanTestCase):
         run_hook = []
         response = app.get('/sub/sub/')
         assert response.status_int == 200
-        assert response.body == 'Deep inside here!'
+        assert response.body == b_('Deep inside here!')
 
         assert len(run_hook) == 3
         assert run_hook[0] == 'before'
@@ -288,7 +317,7 @@ class TestHooks(PecanTestCase):
         app = TestApp(make_app(RootController(), hooks=[SimpleHook(1)]))
         response = app.get('/')
         assert response.status_int == 200
-        assert response.body == 'Hello, World!'
+        assert response.body == b_('Hello, World!')
 
         assert len(run_hook) == 4
         assert run_hook[0] == 'on_route1'
@@ -300,7 +329,7 @@ class TestHooks(PecanTestCase):
 
         response = app.get('/sub/')
         assert response.status_int == 200
-        assert response.body == 'Inside here!'
+        assert response.body == b_('Inside here!')
 
         assert len(run_hook) == 6
         assert run_hook[0] == 'on_route1'
@@ -344,7 +373,7 @@ class TestTransactionHook(PecanTestCase):
 
         response = app.get('/')
         assert response.status_int == 200
-        assert response.body == 'Hello, World!'
+        assert response.body == b_('Hello, World!')
 
         assert len(run_hook) == 3
         assert run_hook[0] == 'start_ro'
@@ -355,7 +384,7 @@ class TestTransactionHook(PecanTestCase):
 
         response = app.post('/')
         assert response.status_int == 200
-        assert response.body == 'Hello, World!'
+        assert response.body == b_('Hello, World!')
 
         assert len(run_hook) == 4
         assert run_hook[0] == 'start'
@@ -450,7 +479,7 @@ class TestTransactionHook(PecanTestCase):
 
         response = app.get('/')
         assert response.status_int == 200
-        assert response.body == 'Index Method!'
+        assert response.body == b_('Index Method!')
 
         assert len(run_hook) == 3
         assert run_hook[0] == 'start_ro'
@@ -461,7 +490,7 @@ class TestTransactionHook(PecanTestCase):
 
         response = app.post('/')
         assert response.status_int == 200
-        assert response.body == 'Index Method!'
+        assert response.body == b_('Index Method!')
 
         assert len(run_hook) == 5
         assert run_hook[0] == 'start'
@@ -474,7 +503,7 @@ class TestTransactionHook(PecanTestCase):
 
         response = app.get('/decorated')
         assert response.status_int == 200
-        assert response.body == 'Decorated Method!'
+        assert response.body == b_('Decorated Method!')
 
         assert len(run_hook) == 7
         assert run_hook[0] == 'start_ro'
@@ -579,7 +608,7 @@ class TestTransactionHook(PecanTestCase):
 
         response = app.get('/')
         assert response.status_int == 200
-        assert response.body == 'Hello, World!'
+        assert response.body == b_('Hello, World!')
 
         assert len(run_hook) == 3
         assert run_hook[0] == 'start_ro'
@@ -592,7 +621,7 @@ class TestTransactionHook(PecanTestCase):
 
         response = app.post('/')
         assert response.status_int == 200
-        assert response.body == 'Hello, World!'
+        assert response.body == b_('Hello, World!')
 
         assert len(run_hook) == 4
         assert run_hook[0] == 'start'
@@ -822,7 +851,7 @@ class TestTransactionHook(PecanTestCase):
 
         response = app.get('/')
         assert response.status_int == 200
-        assert response.body == 'Hello, World!'
+        assert response.body == b_('Hello, World!')
 
         assert len(run_hook) == 6
         assert run_hook[0] == 'start_ro'
@@ -838,7 +867,7 @@ class TestTransactionHook(PecanTestCase):
 
         response = app.post('/')
         assert response.status_int == 200
-        assert response.body == 'Hello, World!'
+        assert response.body == b_('Hello, World!')
 
         assert len(run_hook) == 4
         assert run_hook[0] == 'start'
@@ -947,7 +976,7 @@ class TestTransactionHook(PecanTestCase):
 
         response = app.get('/generic')
         assert response.status_int == 200
-        assert response.body == 'generic get'
+        assert response.body == b_('generic get')
         assert len(run_hook) == 6
         assert run_hook[0] == 'start_ro'
         assert run_hook[1] == 'clear'
@@ -965,7 +994,7 @@ class TestTransactionHook(PecanTestCase):
 
         response = app.post('/generic')
         assert response.status_int == 200
-        assert response.body == 'generic post'
+        assert response.body == b_('generic post')
         assert len(run_hook) == 4
         assert run_hook[0] == 'start'
         assert run_hook[1] == 'inside'
@@ -1052,7 +1081,7 @@ class TestRequestViewerHook(PecanTestCase):
         out = _stdout.getvalue()
 
         assert response.status_int == 200
-        assert response.body == 'Hello, World!'
+        assert response.body == b_('Hello, World!')
         assert 'path' in out
         assert 'method' in out
         assert 'status' in out
@@ -1117,7 +1146,7 @@ class TestRequestViewerHook(PecanTestCase):
         out = _stdout.getvalue()
 
         assert response.status_int == 200
-        assert response.body == 'Hello, World!'
+        assert response.body == b_('Hello, World!')
         assert '/' in out
         assert 'path' in out
         assert 'method' not in out
@@ -1152,7 +1181,7 @@ class TestRequestViewerHook(PecanTestCase):
         out = _stdout.getvalue()
 
         assert response.status_int == 200
-        assert response.body == 'Hello, World!'
+        assert response.body == b_('Hello, World!')
         assert out == ''
 
     def test_item_not_in_defaults(self):
@@ -1179,7 +1208,7 @@ class TestRequestViewerHook(PecanTestCase):
         out = _stdout.getvalue()
 
         assert response.status_int == 200
-        assert response.body == 'Hello, World!'
+        assert response.body == b_('Hello, World!')
         assert 'date' in out
         assert 'method' not in out
         assert 'status' not in out
